#!/usr/bin/env python3
# -*- coding:utf-8 -*-
"""
@file     python_common_tests.py

@author  Lars Pastewka <lars.pastewka@imtek.uni-freiburg.de>

@date    18 Oct 2019

@brief   test common muGrid functionality

Copyright © 2018 Till Junge

µGrid is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License as
published by the Free Software Foundation, either version 3, or (at
your option) any later version.

µGrid is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with µGrid; see the file COPYING. If not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

Additional permission under GNU GPL version 3 section 7

If you modify this Program, or any covered work, by linking or combining it
with proprietary FFT implementations or numerical libraries, containing parts
covered by the terms of those libraries' licenses, the licensors of this
Program grant you additional permission to convey the resulting work.
"""

import unittest
import numpy as np

import muGrid
import _muGrid


class DynCcoordCheck(unittest.TestCase):
    def test_dyn_ccoord_3d(self):
        c = muGrid.DynCcoord([1, 20, 3])
        self.assertEqual(c[0], 1)
        self.assertEqual(c[1], 20)
        self.assertEqual(c[2], 3)
        with self.assertRaises(IndexError):
            c[3]


class OptionDictionaryCheck(unittest.TestCase):
    def test_simple_int(self):
        option_dict = muGrid.Dictionary("int", 1)
        self.assertEqual(option_dict["int"], 1)
        self.assertTrue(isinstance(option_dict["int"], int))

    def test_simple_real(self):
        option_dict = muGrid.Dictionary("real", 1.2)
        self.assertEqual(option_dict["real"], 1.2)
        self.assertTrue(isinstance(option_dict["real"], float))

    def test_simple_matrix(self):
        test_mat = np.random.random((2, 3))
        option_dict = muGrid.Dictionary("matrix", test_mat)
        self.assertEqual(np.linalg.norm(option_dict["matrix"] - test_mat), 0.0)
        self.assertTrue(isinstance(option_dict["matrix"], np.ndarray))

    def test_simple_dict(self):
        d = dict(int=1, float=2.2, matrix=np.eye(2))
        option_dict = muGrid.Dictionary(d)
        self.assertEqual(option_dict["int"], 1)
        self.assertTrue(isinstance(option_dict["int"], int))
        self.assertEqual(option_dict["float"], 2.2)
        self.assertTrue(isinstance(option_dict["float"], float))
        self.assertEqual(np.linalg.norm(option_dict["matrix"] - np.eye(2)), 0.0)
        self.assertTrue(isinstance(option_dict["matrix"], np.ndarray))

    def test_nested_dict(self):
        d = dict(
            int1=1,
            int2=24,
            myfloat=2.2,
            yourfloat=-24.8,
            matrix=np.eye(2),
            dict=dict(int2=2, float2=4.4),
        )
        option_dict = muGrid.Dictionary(d)
        self.assertEqual(option_dict["int1"], 1)
        self.assertTrue(isinstance(option_dict["int1"], int))
        self.assertEqual(option_dict["int2"], 24)
        self.assertTrue(isinstance(option_dict["int2"], int))
        self.assertEqual(option_dict["myfloat"], 2.2)
        self.assertTrue(isinstance(option_dict["myfloat"], float))
        self.assertEqual(option_dict["yourfloat"], -24.8)
        self.assertTrue(isinstance(option_dict["yourfloat"], float))
        self.assertEqual(np.linalg.norm(option_dict["matrix"] - np.eye(2)), 0.0)
        self.assertTrue(isinstance(option_dict["matrix"], np.ndarray))
        with self.assertRaises(RuntimeError):
            isinstance(option_dict["dict"], muGrid.Dictionary)

    def test_simple_reject(self):
        with self.assertRaises(TypeError):
            option_dict = muGrid.Dictionary("string", "clown")

    def test_add(self):
        option_dict = muGrid.Dictionary()
        option_dict.add("int", 2)
        self.assertEqual(option_dict["int"], 2)
        self.assertTrue(isinstance(option_dict["int"], int))
        option_dict.add("real", 3.5)
        self.assertEqual(option_dict["real"], 3.5)
        self.assertTrue(isinstance(option_dict["real"], float))
        option_dict.add("matrix", np.eye(2))
        self.assertEqual(np.linalg.norm(option_dict["matrix"] - np.eye(2)), 0.0)
        self.assertTrue(isinstance(option_dict["matrix"], np.ndarray))

        with self.assertRaises(RuntimeError):
            option_dict.add("int", 3)

    def test_assign(self):
        option_dict = muGrid.Dictionary("int", 1)
        self.assertEqual(option_dict["int"], 1)
        self.assertTrue(isinstance(option_dict["int"], int))

        option_dict["int"] = 2
        self.assertEqual(option_dict["int"], 2)
        self.assertTrue(isinstance(option_dict["int"], int))

        option_dict["int"] = 3.5
        self.assertEqual(option_dict["int"], 3.5)
        self.assertTrue(isinstance(option_dict["int"], float))

        option_dict["int"] = np.eye(2)
        self.assertEqual(np.linalg.norm(option_dict["int"] - np.eye(2)), 0.0)
        self.assertTrue(isinstance(option_dict["int"], np.ndarray))


class NumpyCheck(unittest.TestCase):
    def test_numpy_copy(self):
        nb_grid_pts = (3, 4, 5)
        fc = muGrid.GlobalFieldCollection(nb_grid_pts)
        for component_shape in [(), (1,), (1, 1), (2, 3), (2, 3, 1), (1, 2, 3)]:
            shape = component_shape + nb_grid_pts
            a = np.random.random(shape)
            b = _muGrid.test_numpy_copy(fc, a)
            np.testing.assert_array_equal(b.shape, a.shape)
            np.testing.assert_array_equal(a, b)
            assert a.flags.owndata
            assert b.flags.owndata  # Since this is a copy


if __name__ == "__main__":
    unittest.main()



import numpy as np
from muGrid import GlobalFieldCollection

# Two dimensional grid
nb_grid_pts = (11, 12)
fc = GlobalFieldCollection(nb_grid_pts)

# Get a scalar (real-valued) field
# This call creates a new field if it does not exist.
field = fc.real_field('my-real-valued-field')

# Check the registered fields
assert fc.field_names == ['my-real-valued-field']

# Fill the field with random numbers
field.p = np.random.rand(*nb_grid_pts)

# Set a specific value
field.p[5, 6] = 42

# Get the field, but now into a different variable
field2 = fc.real_field('my-real-valued-field')

# Check that the value is the same
assert field2.p[5, 6] == 42



import numpy as np
from muGrid import GlobalFieldCollection

# Two dimensional grid
nb_grid_pts = (11, 12, 13)
fc = GlobalFieldCollection(nb_grid_pts, sub_pts={'element': 5})

# Get a tensor-field (for example to represent the strain)
strain = fc.real_field(
    'strain',  # name of the field
    (3, 3),  # shape of components
    'element'  # sub-point type
)

# Fill the field with random numbers
strain.s = np.random.rand(*((3, 3, 5) + nb_grid_pts))

# Note that the covenience accessor `p` yields the same field but a different shape
strain.p = np.random.rand(*((3, 3 * 5) + nb_grid_pts))



#!/usr/bin/env python3
# -*- coding:utf-8 -*-
"""
@file   __init__.py

@author Lars Pastewka <lars.pastewka@imtek.uni-freiburg.de>

@date   10 Oct 2019

@brief  Main entry point for muGrid Python module

Copyright © 2018 Till Junge

µGrid is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License as
published by the Free Software Foundation, either version 3, or (at
your option) any later version.

µGrid is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with µGrid; see the file COPYING. If not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

Additional permission under GNU GPL version 3 section 7

If you modify this Program, or any covered work, by linking or combining it
with proprietary FFT implementations or numerical libraries, containing parts
covered by the terms of those libraries' licenses, the licensors of this
Program grant you additional permission to convey the resulting work.
"""

try:
    from mpi4py import MPI
except ModuleNotFoundError:
    MPI = None

import _muGrid

has_mpi = _muGrid.Communicator.has_mpi
if has_mpi and MPI is None:
    raise RuntimeError("MPI support is enabled for muGrid but mpi4py is not available.")

from _muGrid import (get_domain_ccoord, get_domain_index, Pixel, StorageOrder, SubPt, DynCcoord, DynRcoord, IterUnit,
                     Verbosity, GlobalFieldCollection, LocalFieldCollection, Unit, Dictionary)

# FileIONetCDF is only compiled into the library if NetCDF libraries exist
if hasattr(_muGrid, 'FileIONetCDF'):
    OpenMode = _muGrid.FileIONetCDF.OpenMode
    def FileIONetCDF(file_name, open_mode=OpenMode.Read, communicator=None):
        """
        This function is used to open a NetCDF file with a specified mode and
        communicator.

        NetCDF (Network Common Data Form) is a set of software libraries
        and machine-independent data formats that support the creation, access,
        and sharing of array-oriented scientific data.

        Parameters
        ----------
        file_name : str
            The name of the NetCDF file to be opened. This should include the full
            path if the file is not in the current working directory.
        open_mode : OpenMode, optional
            The mode in which the file is to be opened. This should be a value from
            the OpenMode enumeration (Read, Write, Overwrite or Append).
            (Default: OpenMode.Read)
        communicator : Communicator, optional
            The MPI communicator to be used for parallel I/O. If this is not
            provided, the file I/O operations will be serial. (Default: None)

        Returns
        -------
        file : FileIONetCDF
            Returns a FileIONetCDF object which represents the opened file. This object
            can be used to read data from or write data to the file, depending on the
            open mode.
        """
        return _muGrid.FileIONetCDF(file_name, open_mode, Communicator(communicator))
else:
    def FileIONetCDF(*args, **kwargs):
        raise ModuleNotFoundError("muGrid was installed without netCDF support")

from .Parallel import Communicator

__version__ = _muGrid.version.description()
